# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright DinkoO(C) 2012
# Dinko Osmankovic
# dinko.osmankovic@etf.unsa.ba
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('fuelexpensestracker')

from gi.repository import Gtk # pylint: disable=E0611
import logging
logger = logging.getLogger('fuelexpensestracker')

from fuelexpensestracker_lib import Window
from fuelexpensestracker.AboutFuelexpensestrackerDialog import AboutFuelexpensestrackerDialog
from fuelexpensestracker.PreferencesFuelexpensestrackerDialog import PreferencesFuelexpensestrackerDialog
from fuelexpensestracker.GridviewDialog import GridviewDialog
from fuelexpensestracker.ViewpredictionsDialog import ViewpredictionsDialog

import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from backend_gtk3agg import FigureCanvasGTK3Agg as FigureCanvas
from backend_gtk3 import NavigationToolbar2GTK3 as NavigationToolbar
#from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
import random
import time
from numpy import arange, sin, pi

import sqlite3
import datetime
from matplotlib.dates import *
import numpy


def on_pick3(event):
    ind = matplotlib.pyplot.event.ind
    print 'scatter: ', event

# See fuelexpensestracker_lib.Window.py for more details about how this class works
class FuelexpensestrackerWindow(Window):
    __gtype_name__ = "FuelexpensestrackerWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(FuelexpensestrackerWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutFuelexpensestrackerDialog
        self.PreferencesDialog = PreferencesFuelexpensestrackerDialog
        self.GridviewDialog = GridviewDialog
        self.ViewpredictionsDialog = ViewpredictionsDialog

        self.fuel_data_list = self.builder.get_object('fuel_data')
        radioPrice = self.builder.get_object('radioPrice')
        radioConsumption = self.builder.get_object('radioConsumption')
        radioDistance = self.builder.get_object('radioDistance')
        radioPrice.set_active(True)

        radioPrice.connect("toggled", self.callback, "radioPrice")
        radioConsumption.connect("toggled", self.callback, "radioConsumption")
        radioDistance.connect("toggled", self.callback, "radioDistance")

        # Code for other initialization actions should be added here.

        self.fuel_data = [];
        self.graph = None
        self.dates = [];
        self.prices = [];
        self.fuel_consumption = [];
        self.distances = [];

        self.database_connection = None

        try:
            self.call_initialization()

        except ValueError:
            if (len(self.fuel_data) < 1):
                md = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.OK, "No entries in database! Please go to grid view and add some data! ")
                md.run()
                md.destroy()
        
        finally: 
            if self.database_connection:
                self.database_connection.close()

    def call_initialization(self):
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            #cur = database_connection.cursor()    
            #cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('CREATE TABLE fuel_entries (id INTEGER PRIMARY KEY, date VARCHAR(50), at_distance REAL, volume REAL, price REAL)')
            except sqlite3.Error, e:
                pass
                
            #data = cur.fetchone()
            #print "SQLite version: %s" % data

            cur.execute('SELECT * FROM fuel_entries')    
            data = cur.fetchall()

            for row in data:
                self.fuel_data.append([row[0],row[1], row[2], row[3], row[4]])
                self.prices.append(row[4])
                self.dates.append(datetime.datetime.strptime(row[1] ,"%d.%m.%Y"))

            for i in range(1,len(self.fuel_data)):
                self.fuel_consumption.append( (self.fuel_data[i][2]-self.fuel_data[i-1][2])/float(self.fuel_data[i][3]) )               
                self.distances.append( self.fuel_data[i][2] - self.fuel_data[i-1][2] )
                
            self.f = Figure(figsize=(4,3), dpi=72)
            
            self.a = self.f.add_subplot(111)
     
            #x = range(0,len(self.prices))
            x = self.dates
            y = self.prices

            x_min, x_max = min(x), max(x)
            y_min, y_max = min(y), max(y)    
            #print x, y
            self.graph, = self.a.plot(x,y,'o-')
            delta1 = datetime.timedelta(days=1)
            self.a.axis([self.dates[0]-delta1, self.dates[-1]+delta1 , y_min-0.1*(y_max-y_min), y_max+0.1*(y_max-y_min)]);
            self.a.grid(True)
            self.a.set_xlabel('Date')
            self.a.set_ylabel('Price')
            self.a.set_title('Cost of refills')

            years = YearLocator()
            months = MonthLocator()
            days = DayLocator()
            format_date = DateFormatter("%d.%m.%Y")
            self.a.xaxis.set_major_locator(months)
            self.a.xaxis.set_minor_locator(days)
            self.a.xaxis.set_major_formatter(format_date)   
            d = x[-1]-x[0]
            delta = datetime.timedelta(days=d.days/2)
            if (len(x) >= 4):            
                self.a.xaxis.set_ticks([x[0], x[0]+delta ,x[-1]])                   
            else:
                self.a.xaxis.set_ticks([x[0],x[-1]])                  
            
            #f.draw()
            #self.a.patch.set_alpha(0.0)
             
            # Create the widget, a FigureCanvas containing our Figure
            #canvas = FigureCanvas(f)
            #self.ui.scrolledwindow1.set_border_width(10)
            
            self.canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
            self.toolbar = NavigationToolbar(self.canvas, self)
            self.canvas.mpl_connect('pick_event', self.on_pick)
            #canvas.set_size_request(640,480)
            self.ui.scrolledwindow1.add_with_viewport (self.canvas)
            #self.ui.scrolledwindow1.add_with_viewport (self.toolbar)
            self.ui.vbox1.pack_start(self.toolbar, False, False,0)

            vbox_2 = Gtk.VBox(orientation=Gtk.Orientation.VERTICAL)
            #vbox_2 = self.builder.get_object('box2')
            #vbox_2.pack_start(self.canvas, False, False, 0)
            #vbox_2.pack_start(self.toolbar, False, False, 0)
            #self.ui.grid1.add(vbox_2)
#            self.ui.box2.pack_start(self.toolbar)
            self.canvas.draw()
            #self.ui.vbox1.pack_start(canvas, True, False,2)


            
            self.ui.fuelexpensestracker_window.show_all()

    def update_graph(self):
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            #cur = database_connection.cursor()    
            #cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('CREATE TABLE fuel_entries (id INTEGER PRIMARY KEY, date VARCHAR(50), at_distance REAL, volume REAL, price REAL)')
            except sqlite3.Error, e:
                pass
                
            #data = cur.fetchone()
            #print "SQLite version: %s" % data

            cur.execute('SELECT * FROM fuel_entries')    
            data = cur.fetchall()

            self.fuel_data = [];
            self.dates = [];
            self.prices = [];
            self.fuel_consumption = [];
            self.distances = [];

            for row in data:
                self.fuel_data.append([row[0],row[1], row[2], row[3], row[4]])
                self.prices.append(row[4])
                self.dates.append(datetime.datetime.strptime(row[1] ,"%d.%m.%Y"))

            for i in range(1,len(self.fuel_data)):
                self.fuel_consumption.append( (self.fuel_data[i][2]-self.fuel_data[i-1][2])/float(self.fuel_data[i][3]) )               
                self.distances.append( self.fuel_data[i][2] - self.fuel_data[i-1][2] )
                
            x = self.dates
            y = self.prices

            if self.graph == None:
                self.call_initialization()


            self.graph.set_xdata(x)
            self.graph.set_ydata(y)
            self.a.clear()    

            x_min, x_max = min(x), max(x)
            y_min, y_max = min(y), max(y)    
            #print x, y
            self.a.plot(x,y,'bo-')
            delta1 = datetime.timedelta(days=1)
            self.a.axis([self.dates[0]-delta1, self.dates[-1]+delta1 , y_min-0.1*(y_max-y_min), y_max+0.1*(y_max-y_min)]);
            self.a.grid(True)
            self.a.set_xlabel('Date')
            self.a.set_ylabel('Price')
            self.a.set_title('Price graph')

            years = YearLocator()
            months = MonthLocator()
            days = DayLocator()
            format_date = DateFormatter("%d.%m.%Y")
            self.a.xaxis.set_major_locator(months)
            self.a.xaxis.set_minor_locator(days)
            self.a.xaxis.set_major_formatter(format_date)
            d = x[-1]-x[0]
            delta = datetime.timedelta(days=d.days/2)
            if (len(x) >= 4):            
                self.a.xaxis.set_ticks([x[0], x[0]+delta ,x[-1]])                   
            else:
                self.a.xaxis.set_ticks([x[0],x[-1]])                   
                
                #self.f.draw()
                 
                # Create the widget, a FigureCanvas containing our Figure
                #canvas = FigureCanvas(f)
                #self.ui.scrolledwindow1.set_border_width(10)
                
                #self.canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
                #canvas.set_size_request(640,480)
                #self.ui.scrolledwindow1.add_with_viewport (self.canvas)
                
            self.canvas.draw()
                #self.ui.vbox1.pack_start(canvas, True, False,2)
                
                #self.ui.fuelexpensestracker_window.show_all()              

    def callback (self, widget, data=None):
        #print "%s was toggled %s" % (data, ("OFF", "ON")[widget.get_active()])
        #print data, widget.get_active()

        if (widget.get_active()):
            if (data == "radioPrice"):

                if len(self.fuel_data) < 1:
                    md = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "No entries in database. At least 1 is needed for viewing Distance graph.")
                    md.run()
                    md.destroy()
                else:
 
                    x = self.dates
                    y = self.prices


                    self.graph.set_xdata(x)
                    self.graph.set_ydata(y)
                    self.a.clear()    

                    x_min, x_max = min(x), max(x)
                    y_min, y_max = min(y), max(y)    
                    #print x, y
                    self.a.plot(x,y,'bo-')
                    delta1 = datetime.timedelta(days=1)
                    self.a.axis([self.dates[0]-delta1, self.dates[-1]+delta1 , y_min-0.1*(y_max-y_min), y_max+0.1*(y_max-y_min)]);
                    self.a.grid(True)
                    self.a.set_xlabel('Date')
                    self.a.set_ylabel('Price')
                    self.a.set_title('Price graph')

                    years = YearLocator()
                    months = MonthLocator()
                    days = DayLocator()
                    format_date = DateFormatter("%d.%m.%Y")
                    self.a.xaxis.set_major_locator(months)
                    self.a.xaxis.set_minor_locator(days)
                    self.a.xaxis.set_major_formatter(format_date)   
                    d = x[-1]-x[0]
                    delta = datetime.timedelta(days=d.days/2)
                    if (len(x) >= 4):            
                        self.a.xaxis.set_ticks([x[0], x[0]+delta ,x[-1]])                   
                    else:
                        self.a.xaxis.set_ticks([x[0],x[-1]])                  
                    
                    #self.f.draw()
                     
                    # Create the widget, a FigureCanvas containing our Figure
                    #canvas = FigureCanvas(f)
                    #self.ui.scrolledwindow1.set_border_width(10)
                    
                    #self.canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
                    #canvas.set_size_request(640,480)
                    #self.ui.scrolledwindow1.add_with_viewport (self.canvas)
                    
                    self.canvas.draw()
                    #self.ui.vbox1.pack_start(canvas, True, False,2)
                    
                    #self.ui.fuelexpensestracker_window.show_all()
            elif (data == "radioConsumption"):

                if len(self.fuel_data) < 2:
                    md = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "Only 1 entry in database. At least 2 are needed for viewing Consumption graph.")
                    md.run()
                    md.destroy()
 
                else:
                    x = self.dates[1:len(self.dates)]
                    y = self.fuel_consumption


                    self.graph.set_xdata(x)
                    self.graph.set_ydata(y) 
                    self.a.clear()   

                    x_min, x_max = min(x), max(x)
                    y_min, y_max = min(y), max(y)    
                    #print x, y
                    self.a.plot(x,y,'ro-')
                    delta1 = datetime.timedelta(days=1)
                    self.a.axis([x[0]-delta1, x[-1]+delta1 , y_min-0.1*(y_max-y_min), y_max+0.1*(y_max-y_min)]);
                    self.a.grid(True)
                    self.a.set_xlabel('Date')
                    self.a.set_ylabel('Consumption')
                    self.a.set_title('Average consumption between refills')

                    years = YearLocator()
                    months = MonthLocator()
                    days = DayLocator()
                    format_date = DateFormatter("%d.%m.%Y")
                    self.a.xaxis.set_major_locator(months)
                    self.a.xaxis.set_minor_locator(days)
                    self.a.xaxis.set_major_formatter(format_date)   
                    d = x[-1]-x[0]
                    delta = datetime.timedelta(days=d.days/2)
                    if (len(x) >= 4):            
                        self.a.xaxis.set_ticks([x[0], x[0]+delta ,x[-1]])                   
                    else:
                        self.a.xaxis.set_ticks([x[0],x[-1]])                   
                    
                    #self.f.draw()
                     
                    # Create the widget, a FigureCanvas containing our Figure
                    #canvas = FigureCanvas(f)
                    #self.ui.scrolledwindow1.set_border_width(10)
                    
                    #self.canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
                    #canvas.set_size_request(640,480)
                    #self.ui.scrolledwindow1.add_with_viewport (self.canvas)
                    self.canvas.draw()
                    #self.ui.vbox1.pack_start(canvas, True, False,2)
                    
                    #self.ui.fuelexpensestracker_window.show_all()
            elif (data == "radioDistance"):

                if len(self.fuel_data) < 2:
                    md = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "Only 1 entry in database. At least 2 are needed for viewing Distance graph.")
                    md.run()
                    md.destroy()
                else:

                    x = self.dates[1:len(self.dates)]
                    y = self.distances

                    self.graph.set_xdata(x)
                    self.graph.set_ydata(y)  
                    self.a.clear()  

                    x_min, x_max = min(x), max(x)
                    y_min, y_max = min(y), max(y)    
                    #print x, y
                    self.a.plot(x,y,'go-')
                    delta1 = datetime.timedelta(days=1)
                    self.a.axis([x[0]-delta1, x[-1]+delta1 , y_min-0.1*(y_max-y_min), y_max+0.1*(y_max-y_min)]);
                    self.a.grid(True)
                    self.a.set_xlabel('Date')
                    self.a.set_ylabel('Distance')
                    self.a.set_title('Distance travelled between refills')

                    years = YearLocator()
                    months = MonthLocator()
                    days = DayLocator()
                    format_date = DateFormatter("%d.%m.%Y")
                    self.a.xaxis.set_major_locator(months)
                    self.a.xaxis.set_minor_locator(days)
                    self.a.xaxis.set_major_formatter(format_date)   
                    d = x[-1]-x[0]
                    delta = datetime.timedelta(days=d.days/2)
                    if (len(x) >= 4):            
                        self.a.xaxis.set_ticks([x[0], x[0]+delta ,x[-1]])                   
                    else:
                        self.a.xaxis.set_ticks([x[0],x[-1]])            
                    
                    #self.f.draw()
                     
                    # Create the widget, a FigureCanvas containing our Figure
                    #canvas = FigureCanvas(f)
                    #self.ui.scrolledwindow1.set_border_width(10)
                    
                    #self.canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
                    #canvas.set_size_request(640,480)
                    #self.ui.scrolledwindow1.add_with_viewport (self.canvas)
                    self.canvas.draw()
                    #self.ui.vbox1.pack_start(canvas, True, False,2)
                    
                    #self.ui.fuelexpensestracker_window.show_all()

        else:
            pass
                
    def on_pick(self, event):
        box_points = event.artist.get_bbox().get_points()
        print box_points

    def on_btn_GridView_clicked(self, widget, data=None):
        gridview = GridviewDialog()
#        gridview.connect("close", self.callback, "radioPrice")
        gridview.run()
        gridview.destroy()
        self.update_graph()
        

    def on_btn_ViewPredictions_clicked(self, widget, data=None):
        if len(self.fuel_data) > 2:
            viewprediction = ViewpredictionsDialog()
            viewprediction.run()
            viewprediction.destroy()
        else:
            md = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.OK, "Not enough data for prediction! At least 3 entries are required!")
            md.run()
            md.destroy()



