# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright DinkoO(C) 2012
# Dinko Osmankovic
# dinko.osmankovic@etf.unsa.ba
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611

from fuelexpensestracker_lib.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('fuelexpensestracker')

import sqlite3

class GridviewDialog(Gtk.Dialog):
    __gtype_name__ = "GridviewDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated GridviewDialog object.
        """
        builder = get_builder('GridviewDialog')
        new_object = builder.get_object('gridview_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a GridviewDialog object with it in order to
        finish initializing the start of the new GridviewDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)
        
        self.database_connection = None
        fuel_data = self.builder.get_object('fuel_data')
        treeview = self.builder.get_object('treeview1')
        

        try:
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('CREATE TABLE fuel_entries (id INTEGER PRIMARY KEY, date VARCHAR(50), at_distance REAL, volume REAL, price REAL)')
            except sqlite3.Error, e:
                pass
            
            self.database_connection.commit()
            cur.execute('SELECT * FROM fuel_entries')    
            data = cur.fetchall()
            for row in data:
                fuel_data.append([row[0],row[1], row[2], row[3], row[4]])
            treeview.set_model(fuel_data)
                

        except sqlite3.Error, e:
            print "Error %s:" % e.args[0]
        
        #finally: 
        #    if self.database_connection:
        #        self.database_connection.close()

    def on_btn_Close_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        #pass
        if self.database_connection:
            self.database_connection.close()
        self.hide()

    def on_btn_Add_clicked(self, widget, data=None):
        import datetime
        now = datetime.datetime.now()
        fuel_data = self.builder.get_object('fuel_data')
        treeview = self.builder.get_object('treeview1')
#        treeview.set_model(fuel_data)
        cur = self.database_connection.cursor() 

        try:            
            cur.execute('INSERT INTO fuel_entries VALUES(NULL, ?, 1.0, 1.0, 1.0)',(now.strftime("%d.%m.%Y"),)) 
            self.database_connection.commit()
            self.update_list_view()          
                

        except sqlite3.Error, e:
            print "Error %s:" % e.args[0]


    def on_btn_Remove_clicked(self, widget, data=None):
        fuel_data = self.builder.get_object('fuel_data')
        treeview = self.builder.get_object('treeview1')
#        fuel_data.append([len(fuel_data)+1,"",0.0, 0.0, 0.0])
#        treeview.set_model(fuel_data)
        selection = treeview.get_selection()

        result = selection.get_selected()
        if result: #result could be None
           model, iter = result
        #print model.get_value(iter,0)
        cur = self.database_connection.cursor() 

        try:            
            r_id = model.get_value(iter,0)
            cur.execute('DELETE FROM fuel_entries WHERE id=?', (r_id,))
            self.database_connection.commit()
            self.update_list_view()          
                

        except sqlite3.Error, e:
            print "Error %s:" % e.args[0]
        


    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        pass

    def on_cellrenderertext2_edited(self, widget, path, text):
        fuel_data = self.builder.get_object('fuel_data')
        #fuel_data_list[path][1] = text
        #self.update_list_view()
        try:
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            #cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('UPDATE fuel_entries SET date=? WHERE id=?', (text, fuel_data[path][0]) )
            except sqlite3.Error, e:
                pass
            
            self.database_connection.commit()
            self.update_list_view()
        except sqlite3.Error, e:
            pass

    def on_cellrenderertext3_edited(self, widget, path, text):
        fuel_data = self.builder.get_object('fuel_data')
        #fuel_data_list[path][1] = text
        #self.update_list_view()
        try:
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            #cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('UPDATE fuel_entries SET at_distance=? WHERE id=?', (float(text), fuel_data[path][0]) )
            except sqlite3.Error, e:
                pass
            
            self.database_connection.commit()
            self.update_list_view()
        except sqlite3.Error, e:
            pass
    
    def on_cellrenderertext4_edited(self, widget, path, text):
        fuel_data = self.builder.get_object('fuel_data')
        #fuel_data_list[path][1] = text
        #self.update_list_view()
        try:
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            #cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('UPDATE fuel_entries SET volume=? WHERE id=?', (float(text), fuel_data[path][0]) )
            except sqlite3.Error, e:
                pass
            
            self.database_connection.commit()
            self.update_list_view()
        except sqlite3.Error, e:
            pass

    def on_cellrenderertext5_edited(self, widget, path, text):
        fuel_data = self.builder.get_object('fuel_data')
        #fuel_data_list[path][1] = text
        #self.update_list_view()
        try:
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            #cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('UPDATE fuel_entries SET price=? WHERE id=?', (float(text), fuel_data[path][0]) )
            except sqlite3.Error, e:
                pass
            
            self.database_connection.commit()
            self.update_list_view()
        except sqlite3.Error, e:
            pass

    def update_list_view(self):
        #self.database_connection = None
        fuel_data = self.builder.get_object('fuel_data')
        treeview = self.builder.get_object('treeview1')
        

        try:
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            #cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('CREATE TABLE fuel_entries (id INTEGER PRIMARY KEY, date VARCHAR(50), at_distance REAL, volume REAL, price REAL)')
            except sqlite3.Error, e:
                pass
            
            self.database_connection.commit()
            cur.execute('SELECT * FROM fuel_entries')    
            data = cur.fetchall()
            fuel_data.clear()
            for row in data:
                fuel_data.append([row[0],row[1], row[2], row[3], row[4]])
            treeview.set_model(fuel_data)
                

        except sqlite3.Error, e:
            print "Error %s:" % e.args[0]


if __name__ == "__main__":
    dialog = GridviewDialog()
    dialog.show()
    Gtk.main()
