# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright DinkoO(C) 2012
# Dinko Osmankovic
# dinko.osmankovic@etf.unsa.ba
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611

from fuelexpensestracker_lib.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('fuelexpensestracker')


import sqlite3
import datetime
import time
from scipy import stats


class ViewpredictionsDialog(Gtk.Dialog):
    __gtype_name__ = "ViewpredictionsDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated ViewpredictionsDialog object.
        """
        builder = get_builder('ViewpredictionsDialog')
        new_object = builder.get_object('viewpredictions_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def CalculateLinearRegression(self, x, y):
        slope, intercept, r_value, p_value, std_err = stats.linregress(x,y)
        return slope, intercept

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a ViewpredictionsDialog object with it in order to
        finish initializing the start of the new ViewpredictionsDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)
        
        self.database_connection = None
        fuel_data = []

        try:
            self.database_connection = sqlite3.connect('fuelexpensestracker.db')
            cur = self.database_connection.cursor()    
            cur.execute('SELECT SQLITE_VERSION()')
            try:
                cur.execute('CREATE TABLE fuel_entries (id INTEGER PRIMARY KEY, date VARCHAR(50), at_distance REAL, volume REAL, price REAL)')
                self.database_connection.commit()

            except sqlite3.Error, e:
                pass
            
            
            cur.execute('SELECT * FROM fuel_entries')    
            data = cur.fetchall()
            spending_average = 0.0;
            fuel_consumption = [];
            dates_of_refills = [];
            volumes = [];
            prices = [];
            dates_deltas = [];
            try:
            
                for row in data:
                    fuel_data.append([row[0],row[1], row[2], row[3], row[4]])
                    spending_average = spending_average + row[4]
                    dates_of_refills.append(datetime.datetime.strptime(row[1],"%d.%m.%Y"))
                    volumes.append(row[3])
                    prices.append(row[4])
                
                for i in range(1,len(fuel_data)):
                    fuel_consumption.append( (fuel_data[i][2]-fuel_data[i-1][2])/float(fuel_data[i][3]) )              

                days_between_refills_average = 0.0;
                for i in range(len(dates_of_refills)-1):
                    delta = dates_of_refills[i+1] - dates_of_refills[i]
                    dates_deltas.append(delta.days)
                    days_between_refills_average = days_between_refills_average + delta.days
            
            
            
                spending_average = spending_average / float(len(fuel_data))
                days_between_refills_average = days_between_refills_average / float(len(dates_of_refills)-1)

                avgSpendingEntry = self.builder.get_object('avgSpendingEntry')
                avgSpendingEntry.set_text(str(round(spending_average,2)))

                avgRefillsEntry = self.builder.get_object('avgRefillsEntry')
                avgRefillsEntry.set_text(str(round(days_between_refills_average,2)))

                avgConsumptionEntry = self.builder.get_object('avgConsumptionEntry')
                avgConsumptionEntry.set_text(str(round(sum(fuel_consumption)/len(fuel_consumption),2)))

                # PREDICTIONS

                slope, intercept = self.CalculateLinearRegression( range(len(volumes)), volumes)
                nextVolumeEntry = self.builder.get_object('nextVolumeEntry')
                nextVolumeEntry.set_text(str(round( slope*(len(volumes)+1)+intercept,2) ))

                slope, intercept = self.CalculateLinearRegression( range(len(dates_deltas)), dates_deltas)
                nextDateEntry = self.builder.get_object('nextDateEntry')
                adddays = slope*(len(dates_deltas)+1)+intercept
                daysdelta = datetime.timedelta(days=adddays)
                newdate = dates_of_refills[len(dates_of_refills)-1]+daysdelta
                nextDateEntry.set_text(newdate.strftime("%d.%m.%Y"))


                slope, intercept = self.CalculateLinearRegression( range(len(prices)), prices)
                nextPriceEntry = self.builder.get_object('nextPriceEntry')
                nextPriceEntry.set_text(str( round(slope*(len(prices)+1)+intercept ,2)))

                

                #print fuel_data
            except ValueError:
                md = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.OK, "No entries in database! Please go to grid view and add some data! ")
                md.run()
                md.destroy()

                

        except sqlite3.Error, e:
            print "Error %s:" % e.args[0]
            




    def on_btn_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        #pass
        self.hide()

    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        pass


if __name__ == "__main__":
    dialog = ViewpredictionsDialog()
    dialog.show()
    Gtk.main()
